<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="2.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:bpmn="http://www.omg.org/spec/BPMN/20100524/MODEL"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:hpe="extension_HPE"
    xmlns:uuid="java.util.UUID"
    xmlns:java="http://jcp.org/en/jsr/detail?id=270"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:htb="hpe_activator_to_bpm_Functions"
    exclude-result-prefixes="uuid htb">


    <xsl:output method="xml" encoding="utf-8" indent="yes"/>
    <xsl:strip-space elements="*"/>
    
    <!-- functions -->
<!--
     make_nodeID function return the id of the node based in his typology.
     Params:
         Node: context node.
     Return:
         String: value of the new id.
-->

    <xsl:function name="htb:make_nodeID" as="xs:string">
        <xsl:param  name="node" as="node()"/>
        <xsl:choose>
            <xsl:when test="name($node) = 'Process-Node'">
                <xsl:value-of select="concat('st_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
            </xsl:when>
            <xsl:when test="name($node) = 'Rule-Node'">
                <xsl:value-of select="concat('egR_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="concat('egS_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <!-- global variables -->
    <xsl:variable name="id_casePacket"  as="xs:string" select="concat('id_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
    <xsl:variable name="id_resourceRole"  as="xs:string" select="concat('rs_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
    <xsl:variable name="id_resourceParam"  as="xs:string" select="concat('rsP_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
    <xsl:variable name="id_endMessage"  as="xs:string" select="concat('msg_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>

    <!-- mapping the id for the nodes and sequenceFlows -->
    <xsl:variable name="node_map" as="node()*" >
        <xsl:element name="node">
                <xsl:attribute name="name" select="'Start-Node'"/>
                <xsl:value-of  select="concat('se_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
        </xsl:element>
        <xsl:for-each select="Workflow/Nodes/child::*">
            <xsl:element name="node">
                <xsl:attribute name="name" select="./Name"/>
                <xsl:value-of  select="htb:make_nodeID(current())"/>
            </xsl:element>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="sequenceFlow_map" as="node()*">
        <!-- Create the start-node sequenceFlow -->
        <xsl:element name="sequenceFlow">
                <xsl:variable name="start_node" as="node()" select="Workflow/Start-Node"/>
                <xsl:attribute name="sourceRef" select="$node_map [@name = 'Start-Node']"/>
                <xsl:attribute name="targetRef" select="$node_map [@name = $start_node]"/>
                <xsl:value-of select="concat('sf_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
        </xsl:element>
        <!-- Search for the rest of the nodes -->
        <xsl:for-each select="Workflow/Nodes/child::*">
            <xsl:variable name="node" as="node()" select="current()"/>
            <xsl:choose>
                <xsl:when test="name(current()) = 'Process-Node'">
                    <xsl:element name="sequenceFlow">
                        <xsl:attribute name="sourceRef" select="$node_map [@name = $node/Name]"/>
                        <xsl:choose>
                            <xsl:when test="$node/Next-Node">
                                <xsl:attribute name="targetRef" select="$node_map [@name = $node/Next-Node]"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:attribute name="targetRef" select="concat('ee_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                            </xsl:otherwise>
                        </xsl:choose>
                        <xsl:value-of select="concat('sf_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                    </xsl:element>
                </xsl:when>
                <xsl:when test="name(current()) = 'Rule-Node'">
                    <xsl:variable name="outputs" as="xs:string*">
                        <xsl:sequence select="('True-Next-Node', 'False-Next-Node')"/>
                    </xsl:variable>
                    <xsl:for-each select="$outputs">
                        <xsl:variable name="output" as="xs:string" select="."/>
                        <xsl:element name="sequenceFlow">
                            <xsl:attribute name="sourceRef" select="$node_map [@name = $node/Name]"/>
                            <xsl:attribute name="targetRef" select="$node_map [@name = $node/child::*[name() = $output]]"/>
                            <xsl:value-of select="concat('sf_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                        </xsl:element>
                    </xsl:for-each>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:for-each select="$node/child::* [matches(name(),'Default|Switch')]">
                        <xsl:variable name="switch" as="xs:string" select="."/>
                        <xsl:element name="sequenceFlow">
                            <xsl:attribute name="sourceRef" select="$node_map [@name = $node/Name]"/>
                            <xsl:attribute name="targetRef" select="$node_map [@name = $switch]"/>
                            <xsl:value-of select="concat('sf_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                        </xsl:element>
                   </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
    </xsl:variable>

    <xsl:template match="/">
        <!-- Definitions tag-->
        <bpmn:definitions>
            <xsl:attribute name="id" select="'Definition'"/>
            <xsl:attribute name="targetNamespace" select="concat('hpe.',Workflow/Name)"/>
            <xsl:attribute name="typeLanguage" select="'http://www.w3.org/2001/XMLSchema'"/>
            <xsl:attribute name="expressionLanguage" select="'http://www.w3.org/2001/XMLSchema'"/>

            <!-- Search if we need some resources for workflow roles -->
            <xsl:variable name="wf_resource" as="node()*">
                <xsl:variable name="temp_roles" as="node()*">
                    <xsl:apply-templates select="Workflow/child::* [matches(name(),'-Role$')]"/>
                </xsl:variable>
                <xsl:if test="$temp_roles">
                    <bpmn:resource>
                        <xsl:attribute name="id" select="concat('rs_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                        <xsl:attribute name="name" select="'Workflow Roles'"/>
                        <xsl:copy-of select="$temp_roles"/>
                    </bpmn:resource>
                    <xsl:for-each select="Workflow/child::* [matches(name(),'-Role$')]">
                        <xsl:variable name="context_name" as="xs:string" select="name()"/>
                        <xsl:call-template name="get_roles">
                            <xsl:with-param name="context" select="."/>
                            <xsl:with-param name="id" select="$temp_roles/@id [../@name = $context_name]"/>
                        </xsl:call-template>
                    </xsl:for-each>
                </xsl:if>
            </xsl:variable>

            <!-- Starting to fill the BPMN -->
            <xsl:copy-of select="$wf_resource[name() = 'bpmn:resource'] "/>
            <xsl:if test="Workflow/Nodes/Process-Node/Role">
                <bpmn:resource>
                    <xsl:attribute name="id" select="$id_resourceRole"/>
                    <xsl:attribute name="name" select="'Node_Rol'"/>
                    <xsl:apply-templates select="Workflow/Nodes/Process-Node/child::Role [1]"/>
                </bpmn:resource>
            </xsl:if>
            <!-- Fills categories for the node attributes -->
            <xsl:call-template name="get_categories"/>
            <!-- Search for End Handlers -->
            <xsl:variable name="end_handler">
                <xsl:if test="Workflow/End-Handler">
                    <bpmn:message>
                        <xsl:attribute name="id" select="$id_endMessage"/>
                        <xsl:attribute name="itemRef" select="concat('id_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                    </bpmn:message>
                </xsl:if>
            </xsl:variable>
            <xsl:copy-of select="$end_handler "/>

            <!-- Starts with the process -->
            <bpmn:process>    
                <xsl:attribute name="id" select="concat('pr_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                <xsl:attribute name="name" select="Workflow/Name"/>
                <!-- Fills the Solution -->
                <xsl:apply-templates select="Workflow/Solution"/>
                <!-- Fills the Description -->
                <xsl:apply-templates select="Workflow/Description"/>
                <!-- Fills the Workflow attributes (extensionElements) -->
                <xsl:if test="Workflow/attribute::*">
                    <bpmn:extensionElements>
                        <hpe:workFlow_attributes>
                            <xsl:for-each select="Workflow/attribute::*">
                                <xsl:element name="{concat('hpe:',name())}">
                                    <xsl:value-of select="."/>
                                </xsl:element>
                            </xsl:for-each>
                        </hpe:workFlow_attributes>
                    </bpmn:extensionElements>
                </xsl:if>
                <!-- Fills the initialCasePacket -->
                <xsl:apply-templates select="Workflow/Initial-Case-Packet"/>
                <!-- Fills the CasePacket -->
                <xsl:apply-templates select="Workflow/Case-Packet"/>
                <!-- Fills Lane-->
                <xsl:call-template name="get_lane"/>
                <!-- Fills Nodes-->
                <xsl:apply-templates select="Workflow/Nodes"/>
                <!--Fills the Handlers -->
                <xsl:apply-templates select="Workflow/child::* [matches(name(),'Handler$')]"/>
                <!-- Fills the Process Resources -->
                <xsl:if test="$wf_resource">
                    <bpmn:resourceRole>
                        <bpmn:resourceRef>
                            <xsl:value-of select="$wf_resource/@id"/>
                        </bpmn:resourceRef>
                        <xsl:copy-of select="$wf_resource [name() = 'bpmn:resourceParameterBinding']"/>
                    </bpmn:resourceRole>
                </xsl:if>
            </bpmn:process>

            <!-- Fills the itemDefinitions -->
            <bpmn:itemDefinition>
                <xsl:attribute name="id" select="$id_casePacket"/>
                <xsl:attribute name="structureRef" select="'hpe:casePacket'"/>
                <bpmn:documentation>CasePacket dataObject</bpmn:documentation>
            </bpmn:itemDefinition>
            <xsl:if test="$end_handler">
                 <bpmn:itemDefinition>
                     <xsl:attribute name="id" select="$end_handler/bpmn:message/@itemRef"/>
                    <xsl:attribute name="structureRef" select="'payload_message_direction'"/>
                    <bpmn:documentation>End Handler message</bpmn:documentation>
                </bpmn:itemDefinition>
            </xsl:if>
        </bpmn:definitions>
    </xsl:template>

    <!-- get_resource: builds the resource tag for the found roles -->
    <xsl:template name="get_resource" match="* [matches(name(),'Role$')]">
        <bpmn:resourceParameter>
            <xsl:choose>
                <xsl:when test="name() = 'Role'">
                    <xsl:attribute name="id" select="$id_resourceParam"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:attribute name="id" select="concat('rsP_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:attribute name="isRequired" select="false()"/>
            <xsl:attribute name="name" select="name()"/>
            <xsl:attribute name="type" select="'xs:string'"/>
        </bpmn:resourceParameter>
    </xsl:template>

    <!-- get_roles: build the resourceRol tag for the found roles -->
    <xsl:template name="get_roles">
        <xsl:param name="context" as="node()"/>
        <xsl:param name="id" as="xs:string"/>

        <bpmn:resourceParameterBinding>
            <xsl:attribute name="parameterRef" select="$id"/>
            <bpmn:formalExpression> 
                <xsl:value-of select="$context"/>
            </bpmn:formalExpression> 
        </bpmn:resourceParameterBinding>
    </xsl:template>

    <!-- get_categories: build the category tag for the node attributes -->
    <xsl:template name="get_categories">
        <xsl:variable name="types" as="xs:string*">
            <xsl:sequence select="('disablePersistence','inactive')"/>
        </xsl:variable>
        <xsl:for-each select="$types">
            <bpmn:category>
                <xsl:attribute name="name" select="."/>
                <bpmn:categoryValue>
                    <xsl:attribute name="value" select="'true'"/>
                    <xsl:attribute name="id" select="concat(.,'.true')"/>
                </bpmn:categoryValue>
                <bpmn:categoryValue>
                    <xsl:attribute name="value" select="'false'"/>
                    <xsl:attribute name="id" select="concat(.,'.false')"/>
                </bpmn:categoryValue>
            </bpmn:category>
        </xsl:for-each>
    </xsl:template>

    <!-- get_documentation: build documentation tag for Description or Solution tags -->
    <xsl:template name="get_documentation" match="Description | Solution">
            <bpmn:documentation>
                <xsl:value-of select="concat(name(),': ',current())"/>
            </bpmn:documentation>
    </xsl:template>

    <!-- get_initial_casePacket: build the tags to fill the Initial-Case-Packet -->
    <xsl:template name="get_initialCasePacket" match="Initial-Case-Packet">
        <bpmn:ioSpecification>
            <xsl:attribute name="id" select="concat('io_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
            <xsl:variable name="id_DataInput" as="xs:string" select="concat('di_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
            <bpmn:dataInput>
                <xsl:attribute name="id" select="$id_DataInput"/>
                <xsl:attribute name="itemSubjectRef" select="$id_casePacket"/>
                <bpmn:extensionElements>
                    <hpe:initial_casePacket> 
                        <xsl:for-each select="Variable-Value">
                            <hpe:variable_value>
                                <xsl:attribute name="name" select="@name"/>
                                <xsl:attribute name="value" select="@value"/>
                            </hpe:variable_value>
                        </xsl:for-each>
                    </hpe:initial_casePacket> 
                </bpmn:extensionElements>
            </bpmn:dataInput>
            <bpmn:inputSet>
                <xsl:attribute name="id" select="concat('is_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                <bpmn:dataInputRefs>
                    <xsl:value-of select="$id_DataInput"/>
                </bpmn:dataInputRefs>
            </bpmn:inputSet>
            <bpmn:outputSet>
                <xsl:attribute name="id" select="concat('os_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
            </bpmn:outputSet>
        </bpmn:ioSpecification>
    </xsl:template>

    <!-- get_casePacket: fill the Case-Packet variables -->
    <xsl:template name="get_casePacket" match="Case-Packet">
        <bpmn:dataObject>
            <xsl:attribute name="id" select="concat('do_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
            <xsl:attribute name="name" select="'casePacket'"/>
            <xsl:attribute name="isCollection" select="false()"/>
            <xsl:attribute name="itemSubjectRef" select="$id_casePacket"/>
            <bpmn:documentation>CasePacket Variables</bpmn:documentation>
            <bpmn:extensionElements>
                <hpe:casePacket>
                    <xsl:for-each select="Variable">
                        <hpe:variable>
                            <xsl:attribute name="name" select="@name"/>
                            <xsl:attribute name="type" select="@type"/>
                        </hpe:variable>
                    </xsl:for-each>
                </hpe:casePacket>
            </bpmn:extensionElements>
        </bpmn:dataObject>
    </xsl:template>

    <!-- get_nodes: prepares for getting nodes -->
    <xsl:template name="get_nodes" match="Nodes">
        <xsl:variable name="id" as="xs:string" select="$node_map [@name = 'Start-Node']"/>
        <bpmn:startEvent>
            <xsl:attribute name="id" select="$id"/>
            <xsl:attribute name="name" select="Start"/>
            <bpmn:outgoing>
                <xsl:value-of select="$sequenceFlow_map [@sourceRef = $id ]"/>
            </bpmn:outgoing>
        </bpmn:startEvent>
        <bpmn:sequenceFlow>
            <xsl:attribute name="id" select="$sequenceFlow_map [@sourceRef = $id ]"/>
            <xsl:attribute name="sourceRef" select="$id"/>
            <xsl:attribute name="targetRef" select="$sequenceFlow_map/@targetRef [../@sourceRef = $id ]"/>
        </bpmn:sequenceFlow>
        <xsl:for-each select="/Workflow/Nodes/child::* [matches(name(),'Process|Rule|Switch')]">
            <xsl:call-template name="get_next_node">
                <xsl:with-param name="node" select="current()"/>
            </xsl:call-template>
        </xsl:for-each>
    </xsl:template>

    <!-- get_next_node: calls the getters for the node and sequenceFlow and see if we must to fill a endEvet or not -->
    <xsl:template name="get_next_node">
        <xsl:param name="node" as="node()"/>

        <xsl:variable name="id" as="xs:string" select="$node_map [@name = $node/Name]"/>
        <xsl:call-template name="get_node">
            <xsl:with-param name="node" select="$node"/>
            <xsl:with-param name="id" select="$id"/>
        </xsl:call-template>
        <xsl:call-template name="get_sequenceFlow">
            <xsl:with-param name="id" select="$id"/>
            <xsl:with-param name="node" select="$node"/>
        </xsl:call-template>
        <xsl:if test="substring-before($id,'_') = 'st'">
            <xsl:if test="substring-before($sequenceFlow_map/@targetRef [../@sourceRef = $id],'_') = 'ee'">
                <xsl:call-template name="get_endEvent">
                    <xsl:with-param name="id" select="$sequenceFlow_map/@targetRef [../@sourceRef = $id]"/>
                </xsl:call-template>
            </xsl:if>
        </xsl:if>
    </xsl:template>

    <!-- get_node: fills process/rule/switch node -->
    <xsl:template name="get_node">
        <xsl:param name="id" as="xs:string"/>
        <xsl:param name="node" as="node()"/>
        
        <xsl:variable name="type" as="xs:string">
            <xsl:choose>
                <xsl:when test="substring-before($id,'_') = 'st'">
                    <xsl:value-of select="'serviceTask'"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'exclusiveGateway'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:element name="{concat('bpmn:',$type)}">
            <xsl:attribute name="id" select="$id"/>
            <xsl:attribute name="name" select="$node/Name"/>
            <xsl:choose>
                <xsl:when test="substring-before($id,'_') = 'st'">
                    <xsl:attribute name="implementation" select="$node/Action/Class-Name"/>
                </xsl:when>
                <xsl:when test="substring-before($id,'_') = 'egS'">
                    <xsl:attribute name="gatewayDirection" select="'Diverging'"/>
                    <xsl:attribute name="default" select="$sequenceFlow_map [@sourceRef = $id and @targetRef = $node_map [@name = $node/Default]]"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:attribute name="gatewayDirection" select="'Diverging'"/>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:if test="$node/Description">
                <bpmn:documentation> 
                    <xsl:value-of select="$node/Description"/> 
                </bpmn:documentation>
            </xsl:if>
            <bpmn:extensionElements>
                <hpe:actionParams>
                    <xsl:for-each select="$node/Action/Param">
                        <hpe:param>
                            <xsl:attribute name="name" select="@name"/>
                            <xsl:attribute name="value" select="@value"/>
                        </hpe:param>
                    </xsl:for-each>
                </hpe:actionParams>
            </bpmn:extensionElements>
            <xsl:for-each select="$node/attribute::*">
                <bpmn:categoryValueRef>
                    <xsl:if test="name(.) = 'disablePersistence'"> 
                        <xsl:value-of select="'disablePersistence.true'"/>
                    </xsl:if> 
                    <xsl:if test="name(.) = 'inactive'"> 
                        <xsl:value-of select="'inactive.true'"/>
                    </xsl:if> 
                </bpmn:categoryValueRef>
            </xsl:for-each>
            <xsl:for-each select="$sequenceFlow_map [@targetRef = $id]">
                <bpmn:incoming> 
                    <xsl:value-of select="."/> 
                </bpmn:incoming>
            </xsl:for-each>
            <xsl:for-each select="$sequenceFlow_map [@sourceRef = $id]">
                <bpmn:outgoing> 
                    <xsl:value-of select="."/> 
                </bpmn:outgoing>
            </xsl:for-each>
            <xsl:if test="$node/Role">
                <bpmn:resourceRole>
                    <bpmn:resourceRef> 
                        <xsl:value-of select="$id_resourceRole"/> 
                    </bpmn:resourceRef>
                    <bpmn:resourceParameterBinding>
                        <xsl:attribute name="parameterRef" select="$id_resourceParam"/>
                        <bpmn:formalExpression>
                            <xsl:value-of select="$node/Role"/> 
                        </bpmn:formalExpression>
                    </bpmn:resourceParameterBinding>
                </bpmn:resourceRole>
            </xsl:if>
        </xsl:element>
    </xsl:template>

    <!-- get_sequenceFlow: fills the sequenceFlow -->
    <xsl:template name="get_sequenceFlow">
        <xsl:param name="id" as="xs:string"/>
        <xsl:param name="node" as="node()"/>

        <xsl:choose>
            <xsl:when test="substring-before($id,'_') = 'st'">
                <bpmn:sequenceFlow>
                    <xsl:variable name="sequence_id" as="xs:string" select="$sequenceFlow_map [@sourceRef = $id]"/>
                    <xsl:attribute name="id" select="$sequence_id"/>
                    <xsl:attribute name="targetRef" select="$sequenceFlow_map/@targetRef [../. = $sequence_id]"/>
                    <xsl:attribute name="sourceRef" select="$id"/>
                </bpmn:sequenceFlow>
            </xsl:when>
            <xsl:when test="substring-before($id,'_') = 'egR'">
                <xsl:for-each select="$node/child::* [matches(name(), 'Next-Node$')]">
                    <xsl:variable name="id_next" as="xs:string">
                        <xsl:variable name="temp_next" select="name(.)"/>
                        <xsl:value-of select="$node_map [@name = $node/child::* [name() = $temp_next]]"/>
                    </xsl:variable>
                    <bpmn:sequenceFlow>
                        <xsl:attribute name="id" select="$sequenceFlow_map [@sourceRef = $id and @targetRef = $id_next]"/>
                        <xsl:attribute name="targetRef" select="$id_next"/>
                        <xsl:attribute name="sourceRef" select="$id"/>
                        <bpmn:conditionExpression>
                           <xsl:attribute name="id" select="concat('cp_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                           <xsl:attribute name="xsi:type" select="'bpmn:tFormalExpression'"/>
                           <xsl:attribute name="evaluatesToTypeRef" select="'java:java.lang.Boolean'"/>
                           <xsl:attribute name="language" select="$node/Action/Class-Name"/>
                           <xsl:value-of select="substring-before(name(.),'-')"/>
                       </bpmn:conditionExpression>
                    </bpmn:sequenceFlow>
                </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>
                <bpmn:sequenceFlow>
                    <xsl:attribute name="id" select="$sequenceFlow_map [@sourceRef = $id and @targetRef = $node_map [@name = $node/Default]]"/>
                    <xsl:attribute name="targetRef" select="$node_map [@name = $node/Default]"/>
                    <xsl:attribute name="sourceRef" select="$id"/>
                </bpmn:sequenceFlow>
                <xsl:for-each select="$node/Switch">
                    <xsl:variable name="id_next" as="xs:string">
                        <xsl:variable name="temp_next" select="."/>
                        <xsl:value-of select="$node_map [@name = $temp_next]"/>
                    </xsl:variable>
                    <bpmn:sequenceFlow>
                        <xsl:attribute name="id" select="$sequenceFlow_map [@sourceRef = $id and @targetRef = $id_next]"/>
                        <xsl:attribute name="targetRef" select="$id_next"/>
                        <xsl:attribute name="sourceRef" select="$id"/>
                        <bpmn:conditionExpression>
                           <xsl:attribute name="id" select="concat('ep_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                           <xsl:attribute name="xsi:type" select="'bpmn:tFormalExpression'"/>
                           <xsl:attribute name="evaluatesToTypeRef" select="'java:java.lang.String'"/>
                           <xsl:attribute name="language" select="$node/Action/Class-Name"/>
                           <xsl:value-of select="./@name"/>
                       </bpmn:conditionExpression>
                    </bpmn:sequenceFlow>
                </xsl:for-each>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- get_endEvent: fills the endEvent -->
    <xsl:template name="get_endEvent">
        <xsl:param name="id" as="xs:string"/>

        <bpmn:endEvent>
            <xsl:attribute name="id" select="$id"/>
            <xsl:attribute name="name" select="'End Event'"/>
            <bpmn:incoming>
                <xsl:value-of select="$sequenceFlow_map [@targetRef = $id]"/>
            </bpmn:incoming>
            <bpmn:messageEventDefinition>
                <xsl:attribute name="messageRef" select="$id_endMessage"/>
            </bpmn:messageEventDefinition>
        </bpmn:endEvent>
    </xsl:template>

    <!-- get_handlers: fills the handlers subprocess -->
    <xsl:template name="get_handlers" match="* [matches(name(),'-Handler$')]">
        <xsl:variable name="id_se" as="xs:string" select="concat('se_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
        <xsl:variable name="id_sf_1" as="xs:string" select="concat('sf_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
        <xsl:variable name="id_st" as="xs:string" select="concat('st_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
        <xsl:variable name="id_sf_2" as="xs:string" select="concat('sf_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
        <xsl:variable name="id_ee" as="xs:string" select="concat('ee_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>

        <bpmn:subProcess>
            <xsl:attribute name="id" select="concat('sbp_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
            <xsl:attribute name="name" select="name()"/>
            <xsl:attribute name="triggeredByEvent" select="true()"/>
            <xsl:attribute name="completionQuantity" select="'1'"/>
            <xsl:attribute name="isForCompensation" select="false()"/>
            <xsl:attribute name="startQuantity" select="'1'"/>
            <bpmn:startEvent>
                <xsl:attribute name="id" select="$id_se"/>
                <xsl:attribute name="name" select="concat(substring-before(name(),'-'),' Catcher')"/>
                <xsl:attribute name="isInterrupting" select="true()"/>
                <bpmn:outgoing>
                    <xsl:value-of select="$id_sf_1"/>
                </bpmn:outgoing>
                <bpmn:errorEventDefinition/>
            </bpmn:startEvent>
            <bpmn:sequenceFlow>
                <xsl:attribute name="id" select="$id_sf_1"/>
                <xsl:attribute name="sourceRef" select="$id_se"/>
                <xsl:attribute name="targetRef" select="$id_st"/>
            </bpmn:sequenceFlow>
            <bpmn:serviceTask>
                <xsl:attribute name="id" select="$id_st"/>
                <xsl:attribute name="name" select="Name"/>
                <xsl:attribute name="implementation" select="Class-Name"/>
                <bpmn:extensionElements>
                    <xsl:for-each select="Param">
                        <hpe:param>
                            <xsl:attribute name="name" select="@name"/>
                            <xsl:attribute name="value" select="@value"/>
                        </hpe:param>
                    </xsl:for-each>
                </bpmn:extensionElements>
            </bpmn:serviceTask>
            <bpmn:sequenceFlow>
                <xsl:attribute name="id" select="$id_sf_2"/>
                <xsl:attribute name="sourceRef" select="$id_st"/>
                <xsl:attribute name="targetRef" select="$id_ee"/>
            </bpmn:sequenceFlow>
            <bpmn:endEvent>
                <xsl:attribute name="id" select="$id_ee"/>
                <xsl:attribute name="name" select="concat(substring-before(name(),'-'),' End')"/>
                <bpmn:incoming>
                    <xsl:value-of select="$id_sf_2"/>
                </bpmn:incoming>
                
                <xsl:choose>
                    <xsl:when test="name() = 'Error-Handler'">
                        <bpmn:messageEventDefinition>
                            <xsl:attribute name="messageRef" select="$id_endMessage"/>
                        </bpmn:messageEventDefinition>
                    </xsl:when>
                    <xsl:otherwise>
                        <bpmn:terminateEventDefinition>
                          <xsl:attribute name="id" select="concat('ed_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                        </bpmn:terminateEventDefinition>
                    </xsl:otherwise>
                </xsl:choose>
            </bpmn:endEvent>
        </bpmn:subProcess>
    </xsl:template>

    <!-- get_lane: fills the lane -->
    <xsl:template name="get_lane">
        <bpmn:laneSet>
            <xsl:attribute name="id" select="concat('ls_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
            <bpmn:lane>
                <xsl:attribute name="id" select="concat('la_',substring-before(uuid:toString(uuid:randomUUID()),'-'))"/>
                <xsl:attribute name="name" select="'HPE Activator'"/>
                <xsl:for-each select="$node_map">
                    <bpmn:flowNodeRef>
                        <xsl:value-of select="."/>
                    </bpmn:flowNodeRef>
                </xsl:for-each>
                <xsl:for-each select="$sequenceFlow_map/@targetRef [matches(.,'ee')]">
                    <bpmn:flowNodeRef>
                        <xsl:value-of select="."/>
                    </bpmn:flowNodeRef>
                </xsl:for-each>
            </bpmn:lane>
        </bpmn:laneSet>
    </xsl:template>

</xsl:stylesheet>
