<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="2.0"
    xpath-default-namespace="http://www.omg.org/spec/BPMN/20100524/MODEL"
    xmlns:bth="functions:BPM_to_HPActivator"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:hpe="extension_HPE"
    exclude-result-prefixes="bth hpe xs">

    <xsl:import href="functions_bpm.xsl"/>
    <xsl:output method="xml" encoding="utf-8" indent="yes"/>
    <xsl:strip-space elements="*"/>
    

    <xsl:template match="/definitions">
        <xsl:text disable-output-escaping='yes'>&#xA;&lt;!DOCTYPE Workflow SYSTEM "workflow.dtd"&gt;&#xA;&#xA;</xsl:text>

        <Workflow>
            <!--Searching for attributes-->
            <xsl:variable name="process" as="node()" select="process"/>
            <xsl:for-each select="bth:list_Attributes('workflow')">
                <xsl:variable name="attribute_value" as="xs:string" select="bth:validate_Attribute_workflow($process,current())"/>
                <xsl:if test="$attribute_value != 'false' ">
                    <xsl:attribute name="{current()}" select="$attribute_value"/>
                </xsl:if>
            </xsl:for-each>
            <Name> <xsl:value-of select="process/@name"/> </Name>
            <xsl:for-each select="process/documentation [matches(.,'^Solution: ')]">
                <Solution> <xsl:value-of select="substring-after(.,'Solution: ')"/> </Solution>
            </xsl:for-each>
            <xsl:for-each select="process/documentation [matches(.,'^Description: ')]">
                <xsl:if test="matches(.,'^Description: ')">
                    <Description> <xsl:value-of select="substring-after(.,'Description: ')"/> </Description>
                </xsl:if>
            </xsl:for-each>

            <xsl:call-template name="get_roles"/>
            <Start-Node> <xsl:value-of select="bth:get_startNode(process/startEvent)"/> </Start-Node>
            <Nodes>
                <!-- Creating nodes-->
                <xsl:for-each select="process/child::* [matches(name(),'serviceTask|exclusiveGateway')]" >
                    <xsl:call-template name="node">
                        <xsl:with-param name="node" select="current()"/>
                    </xsl:call-template>
                </xsl:for-each>
            </Nodes>
            <!-- Getting the Handlers -->
            <xsl:apply-templates select="process/subProcess"/>
            <!-- Getting  the Case-Packet -->
            <xsl:apply-templates select="process/dataObject/extensionElements"/>
            <!-- Getting   Initial-Case-Packet -->
            <xsl:apply-templates select="process/ioSpecification/dataInput/extensionElements"/>
        </Workflow>
    </xsl:template>

    <!-- get_roles template: get the roles of the workflow-->
    <xsl:template name="get_roles">
        <xsl:if test="process/../resource/@name = 'Workflow Roles'">
            <xsl:variable name="dir" select="process/../resource/resourceParameter"/>
            <xsl:for-each select="process/resourceRole/resourceParameterBinding">
                <xsl:variable name="for_each" as="node()" select="."/>
                <xsl:element name="{$dir/@name [../@id = $for_each/@parameterRef]}">
                    <xsl:value-of select="."/>
                </xsl:element>
            </xsl:for-each>
        </xsl:if>
    </xsl:template>

    <!-- node template: search  the nodes  in the workflow -->
    <xsl:template name="node">
        <xsl:param name="node" as="node()"/>

        <xsl:choose>
            <xsl:when test="matches(name($node),'task','i')">
                <xsl:call-template name="process_Node">
                    <xsl:with-param name="node" select="$node"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:when test="matches(name($node),'gateway','i') and not($node/@default)">
                <xsl:call-template name="rule_Node">
                    <xsl:with-param name="node" select="$node"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="switch_Node">
                    <xsl:with-param name="node" select="$node"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- process_Node template: creates a process_node in the new  workflow -->
    <xsl:template name="process_Node">
        <xsl:param name="node" as="node()"/>
        <Process-Node>
            <!--Searching for attributes-->
            <xsl:for-each select="bth:list_Attributes('process-node')">
                <xsl:if test="bth:validate_Attribute($node,current())">
                    <xsl:attribute name="{current()}" select="'true'"/>
                </xsl:if>
            </xsl:for-each>
            <!-- We continue filling the node-->
            <Name><xsl:value-of select="$node/@name"/></Name>
            <xsl:if test="$node/documentation">
                <Description><xsl:value-of select="$node/documentation"/></Description>
            </xsl:if>
            <xsl:if test="$node/resourceRole/resourceRef = $node/../../resource/@id">
                <Role> <xsl:value-of select="$node/resourceRole/resourceParameterBinding/formalExpression"/> </Role>
            </xsl:if>
            <Action>
                <Class-Name><xsl:value-of select="$node/@implementation"/></Class-Name>
                <xsl:apply-templates select="$node/extensionElements/hpe:actionParams"/>
            </Action>
            <xsl:variable name="next_node" as="xs:string" select="bth:get_nextNode($node)"/>
            <!-- Check if the next node finish the workflow-->
            <xsl:if test="$next_node != 'endEvent'">
                <Next-Node><xsl:value-of select="$next_node"/></Next-Node>
            </xsl:if>
        </Process-Node>
    </xsl:template>

    <!-- rule_Node template: creates a rule_node in the new  workflow -->
    <xsl:template name="rule_Node">
        <xsl:param name="node" as="node()"/>
        <Rule-Node>
        <!--Searching for attributes-->
        <xsl:for-each select="bth:list_Attributes('gateway-node')">
            <xsl:if test="bth:validate_Attribute($node,current())">
                <xsl:attribute name="{current()}" select="'true'"/>
            </xsl:if>
        </xsl:for-each>
            <!-- We continue filling the node-->
            <Name><xsl:value-of select="$node/@name"/></Name>
            <xsl:if test="$node/documentation">
                <Description><xsl:value-of select="$node/documentation"/></Description>
            </xsl:if>
            <Action>
                <Class-Name><xsl:value-of select="$node/../sequenceFlow/conditionExpression/@language [subsequence($node/outgoing,1,1) = ../../@id] "/></Class-Name>
                <xsl:apply-templates select="$node/extensionElements/hpe:actionParams"/>
            </Action>
            <True-Next-Node><xsl:value-of select="bth:get_nextNode($node,'true')"/></True-Next-Node>
            <False-Next-Node><xsl:value-of select="bth:get_nextNode($node,'false')"/></False-Next-Node>
        </Rule-Node>
    </xsl:template>  
    
    <!-- switch_Node template: creates a rule_node in the new  workflow -->
    <xsl:template name="switch_Node">
        <xsl:param name="node" as="node()"/>
        <Switch-Node>
        <!--Searching for attributes-->
        <xsl:for-each select="bth:list_Attributes('gateway-node')">
            <xsl:if test="bth:validate_Attribute($node,current())">
                <xsl:attribute name="{current()}" select="'true'"/>
            </xsl:if>
        </xsl:for-each>
            <!-- We continue filling the node-->
            <Name><xsl:value-of select="$node/@name"/></Name>
            <xsl:if test="$node/documentation">
                <Description><xsl:value-of select="$node/documentation"/></Description>
            </xsl:if>
            <Action>
                <Class-Name><xsl:value-of select="$node/../sequenceFlow/conditionExpression/@language [subsequence($node/outgoing,1,1) = ../../@id] "/></Class-Name>
                <xsl:apply-templates select="$node/extensionElements/hpe:actionParams"/>
            </Action>
            <xsl:for-each select="$node/outgoing [. != ../@default]"> 
                <Switch><xsl:attribute name="name" select="concat('case',position()-1)"/>
                    <xsl:value-of select="bth:get_nextNode($node,concat('case',position()-1))"/>
                </Switch>
            </xsl:for-each>
            <Default><xsl:value-of select="bth:get_nextNode($node,'default')"/></Default>
        </Switch-Node>
    </xsl:template>

    <!-- get_handler: Fill the Handlers -->
    <xsl:template name="get_handler" match="subProcess [matches(@name,'Error-Handler|End-Handler')]">
        <xsl:element name="{@name}">
            <xsl:if test="serviceTask/@name">
                <Name> <xsl:value-of select="serviceTask/@name"/> </Name>
            </xsl:if>
            <Class-Name> <xsl:value-of select="serviceTask/@implementation"/>  </Class-Name>
            <xsl:apply-templates select="serviceTask/extensionElements"/>
        </xsl:element>
    </xsl:template>

    <!-- hpe:param: Fill the Param tags -->
    <xsl:template name="hpe:param"  match="hpe:param">
        <Param>
            <xsl:attribute name="name" select="@name"/> 
            <xsl:attribute name="value" select="@value"/> 
        </Param>
    </xsl:template>

    <!-- hpe:casePacket: Fill the Case-Packet tag -->
    <xsl:template name="hpe:casePacket" match="hpe:casePacket">
        <Case-Packet>
            <xsl:for-each select="hpe:variable">
                <Variable>
                    <xsl:attribute name="name" select="@name"/> 
                    <xsl:attribute name="type" select="@type"/> 
                    <xsl:if test="@disablePersistence = 'true'">
                        <xsl:attribute name="disablePersistence" select="'true'"/> 
                    </xsl:if>
                </Variable> 
            </xsl:for-each>
        </Case-Packet>
    </xsl:template>

    <!-- hpe:initial_casePacket: Fill the initial_casePacket tags -->
    <xsl:template name="hpe:initial_casePacket"  match="hpe:initial_casePacket">
        <Initial-Case-Packet>
            <xsl:for-each select="hpe:variable_value">
                <Variable-Value>
                    <xsl:attribute name="name" select="@name"/> 
                    <xsl:attribute name="value" select="@value"/> 
                </Variable-Value> 
            </xsl:for-each>
        </Initial-Case-Packet>
    </xsl:template>

</xsl:stylesheet>
