<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="2.0"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xpath-default-namespace="http://www.omg.org/spec/BPMN/20100524/MODEL"
    xmlns:bth="functions:BPM_to_HPActivator"
    xmlns:hpe="extension_HPE"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!--
    validate_Attribute function return the value of the attribute to validate in the node, if the node has not that attribute, the function returns false.
    Params:
        Node: context node.
        String: name of the attribute to validate.
    Return:
        Boolean: value of the attribute.
 -->
    <xsl:function name="bth:validate_Attribute" as="xs:boolean">
        <xsl:param name="context" as="node()"/>
        <xsl:param name="attribute" as="xs:string"/>

        <xsl:variable name="root"  as="node()" select="$context/parent::*/parent::*"/>
        <!-- Searching for attribute in the workflow-->
        <xsl:choose>
            <xsl:when test="$root/category/@name [. = $attribute]">
                <!-- Searching for id of the  attribute-->
                <xsl:variable name="id" as="xs:string" select="$root/category/categoryValue/@id [../@value = 'true' and ../../@name = $attribute]"/> 
                <!-- Searching for attribute in context node-->
                <xsl:choose>
                    <xsl:when test="$context/categoryValueRef [. = $id]">
                        <xsl:value-of select="true()"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="false()"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="false()"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

<!--
    validate_Attribute_wf function return the value of the attribute to validate in the workflow, if the workflow  has not that attribute, the function returns false.
    Params:
        Node: context node.
        String: name of the attribute to validate.
    Return:
        String: value of the attribute.
 -->
    <xsl:function name="bth:validate_Attribute_workflow" as="xs:string">
        <xsl:param name="context" as="node()"/>
        <xsl:param name="attribute" as="xs:string"/>

        <!-- Searching for attribute in context node-->
        <xsl:variable name="attribute_node" as="node()*" select="$context/extensionElements/hpe:workFlow_attributes/child::* [matches(name(),$attribute)]"/>

        <xsl:choose>
            <!-- Getting the value of the attribute-->
            <xsl:when test="$attribute_node">
                <xsl:value-of select="$attribute_node"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="false()"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

<!--
    list_Attribute function return a list with the attributes which can have some type of node.
    Params:
        String: name of the type of node.
    Return:
        String*: list  of the attributes which a node has.
 -->
    <xsl:function name="bth:list_Attributes" as="xs:string*">
        <xsl:param name="typeNode" as="xs:string"/>

        <!-- Choosing the right set of attributes -->
            <xsl:choose>
                <xsl:when test="$typeNode = 'gateway-node'">disablePersistence</xsl:when>
                <xsl:when test="$typeNode = 'process-node'"> <xsl:sequence select="('disablePersistence','inactive')"/> </xsl:when>
                <xsl:when test="$typeNode = 'workflow'"> <xsl:sequence select="('Init-On-Startup', 'Unique', 'statEnabled', 'auditEnabled', 'autoAuditEnabled', 'disablePersistence', 'maxNodesPerThread', 'auditStateChangeEvent')"/> </xsl:when>
            </xsl:choose>
    </xsl:function>
<!--
    get_startNode function return the start node of the  workflow.
    Params:
        Node: the startEvent node.
    Return:
        String: the name of the start node in the workflow.
 -->
    <xsl:function name="bth:get_startNode" as="xs:string">
        <xsl:param name="context" as="node()"/>

        <!-- Choosing the outgoing id for the sequenceFlow -->
        <xsl:variable name="id_out">
            <xsl:choose>
                <xsl:when test="$context/outgoing">
                    <xsl:value-of select="$context/outgoing"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:message terminate="yes"> The node has not outgoing node </xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>    
        <!-- Getting the id of the node from the sequenceFlow -->
        <xsl:variable name="id_sf" as="xs:string" select="$context/../sequenceFlow/@targetRef [../@id = $id_out]"/>
        <xsl:value-of select="$context/../child::*/@name [../@id = $id_sf]"/>
    </xsl:function>

<!--
    get_nextNode function return the name of the next node in the workflow. Only for Process-Node.
    Params:
        Node: the context node.
    Return:
        String: the name of the next node in the workflow.
 -->
    <xsl:function name="bth:get_nextNode" as="xs:string">
        <xsl:param name="context" as="node()"/>

        <!-- Choosing the outgoing id for the sequenceFlow -->
        <xsl:variable name="id_out" as="xs:string">
            <xsl:choose>
                <xsl:when test="$context/outgoing">
                    <xsl:value-of select="$context/outgoing"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:message terminate="yes"> The node has not outgoing node </xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>    
        <!-- Getting the id of the node from the sequenceFlow -->
        <xsl:variable name="id_sf" as="xs:string" select="$context/../sequenceFlow/@targetRef [../@id = $id_out]"/>
        <!-- Return the name of the node and search if it is the final node -->
        <xsl:choose>
            <xsl:when test="name($context/../endEvent [./@id = $id_sf])">
                <xsl:value-of select="'endEvent'"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$context/../child::*/@name [../@id = $id_sf]"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

<!--
    get_nextNode function return the name of the next node in the workflow. Only for gateways.
    Params:
        Node: the context node.
        String: the flow condition.
    Return:
        String: the name of the next node in the workflow.
 -->
    <xsl:function name="bth:get_nextNode" as="xs:string">
        <xsl:param name="context" as="node()"/>
        <xsl:param name="condition" as="xs:string"/>

        <!-- Choosing the outgoing id for the sequenceFlow -->
        <xsl:variable name="id_out" as="node()*">
            <xsl:choose>
                <xsl:when test="$context/outgoing">
                    <xsl:sequence select="$context/outgoing" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:message terminate="yes"> The node has not outgoing node </xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>    
        <!-- Getting the id of the node from the sequenceFlow -->
        <xsl:variable name="id_next" as="xs:string">
            <xsl:choose>
                <!-- Getting the information for the switch node -->
                <xsl:when test="count($id_out) > 2">
                    <xsl:choose>
                        <xsl:when test="$condition='default'">
                            <xsl:value-of select="$context/../sequenceFlow/@targetRef [../@id = $context/@default]"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:for-each select="$id_out">
                                <xsl:variable name="sequenceFlow" as="node()" select="$context/../sequenceFlow [@id = current()]"/>
                                <xsl:if test="matches($sequenceFlow/conditionExpression,$condition,'i')">
                                    <xsl:value-of select="$sequenceFlow/@targetRef"/>
                                </xsl:if>
                            </xsl:for-each>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <!-- Getting the information for the rule node -->
                <xsl:otherwise>
                    <xsl:variable name="first_sequenceFlow" as="node()" select="$context/../sequenceFlow [@id = subsequence($id_out,1,1)]"/>
                    <xsl:choose>
                        <xsl:when test=" matches($first_sequenceFlow/conditionExpression,$condition,'i')">
                            <xsl:value-of select="$first_sequenceFlow/@targetRef"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$context/../sequenceFlow/@targetRef [../@id = subsequence($id_out,2,1)]"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <!-- Return the name of the node -->
        <xsl:value-of select="$context/../child::*/@name [../@id = $id_next]"/>
    </xsl:function>
 
</xsl:stylesheet>
